############################################################
#                                                          #
#              Information                                 #
#                                                          #
############################################################

#Dates: 1/01/2021 24h *de moment*   
#Time zone and hour format: Hour of the day is shown in UTC. Value at Hour 1 correspond to the hour starting at 1 and ending at 2. As April falls in summer time (UTC + 2 hours), to convert from local time to UTC we move two hours backwards. So, 7 am local time is 5 am UTC.   
#Pollutant: NO2   
#Catalunya    
#Sites: 72 monitoring stations
#Observations source: 
#Models intercompared:  
#  Raw model from Caliope
#  LOOCV applied to Caliope
#-------------------------------------------------------------------------------
# Measure the time it takes to run the analysis
start_time <- Sys.time()
#-------------------------------------------------------------------------------
#              LOOCV TO CALIOPE DATA WITH CATALONIA OBSERVATIONS
#-------------------------------------------------------------------------------

############################################################
#                                                          #
#              Set libraries                               #
#                                                          #
############################################################
# ! check the version of the package or R if any library or function does not work correctly

library(urbanR)
library(RNetCDF)
library(sp)
library(sf)
#library(mapview)
library(ncdf4)
library(rgeos) 
library(ggplot2)
library(plyr) #OJO! la llibreria plyr ha de ser cridada abans que dplyr 
library(dplyr) ##
library(broom)
library(magrittr)
library(ggpubr)
library(raster)
library(rgdal)
#library(tmap)
library(gstat)
library(sp)
library(abind)
library(openair)
#library(Metrics)

#-------------------------------------------------------------------------------

## LOAD config_file

args = commandArgs(TRUE)
if (length(args) > 0) {
  configFile_path <- args[1]
} else {
  cat("\n***WARNING: no ConfigFile was provided. Using default ConfigFile.\n")
}

#config_file it's a file where we change the data, where we want to save the results, etc. so now we extract these data from it:
#configFile_path <- "/esarchive/scratch/mhajji/Meriem/RUN_UK_regional/LOOCV/config_file_2021.yml"


config          <- yaml::yaml.load_file(configFile_path)
sim_folder_path <- config$sim_folder_path
obs_path        <- config$obs_path
model_path      <- config$model_path
save_result     <- config$save_result
pollutant       <- config$pollutant
print(paste0("Selected pollutant: ", pollutant))

date_time_end   <- config$date_time_end
date_time_begin <- config$date_time_begin

name            <- config$doc_name
variogram_path  <- config$variogram_path
path_code       <- config$path_code

set_stations       <- config$set_stations
set_observations   <- config$set_observations 
set_model          <- config$set_model
execute_loocv      <- config$execute_loocv
execute_statistics <- config$execute_statistics

save_general_statistics    <- config$save_general_statistics 
save_time_series           <- config$save_time_series 
save_temporal_variation    <- config$save_temporal_variation 


# As we had problems with Ordinary and Universal Kriging variograms, we saved one of each to read them in case we have problems with the one executed for a specific date or hour
variogram_ok <- paste0(variogram_path, "Variogram_OK.rds")
variogram_uk <- paste0(variogram_path, "Variogram_UK.rds")

############################################################
#                                                          #
#              Set Units                                   #
#                                                          #
############################################################
print("Setting units of obs and model")
# We need to adjust the units of the observations to ug/m3, so first we pick up the constant depending on the pollutant that we are working with
pol          <- c("no2","o3")
k            <- c(1.881, 1.963)
# We will use the limit parameter to avoid observations that we consider wrong in loocv loop. As for NO2 the alert is established at 400 ug/m3, and 240 for O3 we will put this number as the boundary
limit        <- c(400, 240) 

position     <- match(pollutant, pol) 
number       <- match(pol[position], pol)

limit        <- limit[number]
obs_constant <- k[number]
mod_constat  <- obs_constant*1000

#pol <- c("O3", "NO2","SO2","NO","CO","NH3","HNO3","HONO","PM10","PM25","wspd10","wdir10","PRES","WD","WS","PM10SO4","PM10NO3","PM10NH4","PM10NA","PM10CL","PM10BC","PM10SOA","PM10PPM","PM10DU","PM10POA","PM10UNSPEC", "EC")
# this is not correct, EC is provided by reduce in kgm3, thats why we multiply by 1e09 to obtain ugm3. hence it is not a ppm to ugm3 conversion
#k   <- c(1962, 1881 ,2620 ,1227,1145,695  ,2576 ,1922 ,1e09 ,1e09 ,1 ,1 ,0.01 ,1 ,1 ,1e09 ,1e09 ,1e09 ,1e09 ,1e09 ,1e09 ,1e09 ,1


############################################################
#                                                          #
#              Set temporal context                        #
#                                                          #
############################################################
print("setting dates")
#date_times et fa un super vector d'una  columna, dia per dia, hora per hora des de l'inici fins a la data final p.ex: 2021-01-01 00:00:00
date_times <- data_frame_dates(date_time_begin = date_time_begin,
                               date_time_end = date_time_end)

#dates: Llista de caracters anymesdia p.ex: 20210101
dates <- unique(paste0(as.character(strftime(date_times[, 1], format="%Y", tz = "UTC")),
                       as.character(strftime(date_times[, 1], format="%m", tz = "UTC")),
                       as.character(strftime(date_times[, 1], format="%d", tz = "UTC"))))
#year_month: igual que date pero anymes p.ex: 202101
year_month <- unique(paste0(as.character(strftime(date_times[, 1], format="%Y", tz = "UTC")),
                            as.character(strftime(date_times[, 1], format="%m", tz = "UTC"))))

############################################################
#                                                          #
#                 Load Local Stations                      #
#                                                          #
############################################################
print("Load Local Stations")
# local_stations: special points data frame with two cols [NO2 and ref_obs-reference of station-], at the moment we have 72 stations in Cat

if (set_stations){
  source(paste0(path_code,"set_stations.R"))
  print("Set stations load: done")
} else {
  local_stations <- readRDS(file = paste0(save_result,"local_stations_",pollutant,".rds"))
  print("Read local stations: done")
}

############################################################
#                                                          #
#                 Load Observations                        #
#                                                          #
############################################################
print("Load observations")
if (set_observations){
  source(paste0(path_code,"load_obs_cat.R"))
  print("Observations load: done")
} else {
  df_obs_all <- readRDS(file = paste0(save_result,"df_obs_",pollutant,".rds"))
  print("Read observations: done")
}

############################################################
#                                                          #
#                 Load Model                               #
#                                                          #
############################################################
print("Load Model")
if (set_model){
  source(paste0(path_code,"load_caliope_cat.R")) 
  print("Raw model load: done")
} else {
  model <- readRDS(file = paste0(save_result,"model_",pollutant,".rds"))
  print("Read raw model: done")
}

############################################################
#                                                          #
#                 Loop Universal Kriging                   #
#                                                          #
############################################################
print("Loop Universal Kriging")
if (execute_loocv){
  source(paste0(path_code,"loop_uk_cat.R"))  
  print("Corrected model load: done")
} else {
  LOOCV_total_df     <- readRDS(file = paste0(save_result,"LOOCV_",pollutant,"_df.rds"))
  llista_uk_total_df <- readRDS(file = paste0(save_result,"llista_uk_",pollutant,"_df.rds"))
  print("Read corrected model: done")
}


############################################################
#                                                          #
#              evaluation statistics                       #
#                                                          #
############################################################
print("Evaluating statistics")

if (execute_statistics){
  
  # Evaluation stadistics
  source(paste0(path_code,"stadistics_uk_cat.R")) 
  print("Evaluation statistics LOOCV vs raw model: done")
  
  #Temporal Variation
  source(paste0(path_code,"temporal_variation_uk_cat.R")) 
  print("Temporal variation: done")
  
  #Time Series
  source(paste0(path_code,"time_series_uk_cat.R")) 
  print("Time Series avaluation: done")
  
  
} else {
  print("Evaluation Statistics not executed")
}

#-------------------------------------------------------------------------------
#Temps que triga en executar-se el fitxer

end_time <- Sys.time()
temps_total <- end_time-start_time
print(temps_total)
